using System;
using System.Collections.Generic;
using System.ComponentModel;
using System.Data;
using System.Drawing;
using System.Linq;
using System.Text;
using System.Threading.Tasks;
using System.Windows.Forms;
using IDTechSDK;

namespace WorldNet_Demo_App_Windows
{
    public partial class Form1 : Form
    {
        public Form1()
        {
            InitializeComponent();
            IDT_Device.setCallbackIP(MessageCallBack);
            Profile.discoverDevice();
        }
        public static String GetTimestamp()
        {
            DateTime value = DateTime.Now;
            return (value.ToString("HH:mm:ss.fff") + "  ");
        }
        delegate void SetTextCallback(string text, string ident);

        private void output(string text, string ident)
        {
            // InvokeRequired required compares the thread ID of the
            // calling thread to the thread ID of the creating thread.
            // If these threads are different, it returns true.
            if (sdkOutput.InvokeRequired)
            {
                try
                {
                    SetTextCallback d = new SetTextCallback(output);
                    Invoke(d, new object[] { text, ident });
                }
                catch
                {

                }

            }
            else
            {
                if (ident == null || ident.Length == 0) ident = IDT_Device._path;
                if (text == null || text.Length == 0)
                {
                    sdkOutput.Text = "";
                    return;
                }
                try
                {
                    sdkOutput.AppendText(GetTimestamp() + "\r\n" +  text);
                }
                catch (Exception)
                {
                }

            }

            text = "";
        }

        private void connectlabel(string text, string ident)
        {
            // InvokeRequired required compares the thread ID of the
            // calling thread to the thread ID of the creating thread.
            // If these threads are different, it returns true.
            if (sdkOutput.InvokeRequired)
            {
                try
                {
                    SetTextCallback d = new SetTextCallback(connectlabel);
                    Invoke(d, new object[] { text, ident });
                }
                catch
                {

                }

            }
            else
            {
                connectionLabel.Text = text;
              

            }

            connectionLabel.Text = text;
        }


        private void MessageCallBack(IDTechSDK.IDT_DEVICE_Types type, DeviceState state, byte[] data,
         IDTTransactionData cardData, EMV_Callback emvCallback, RETURN_CODE transactionResultCode, string ident)
        {

            if (InvokeRequired)
            {
                try
                {

                    Invoke((MethodInvoker)delegate
                    {
                        MessageCallBack(type, state, data, cardData, emvCallback, transactionResultCode, ident);
                    });

                    return;
                }
                catch
                {

                }

            }



            IDTechComm comm = Profile.getComm(type, DEVICE_INTERFACE_Types.DEVICE_INTERFACE_USB, ident);
            if (comm == null) comm = Profile.getComm(type, DEVICE_INTERFACE_Types.DEVICE_INTERFACE_SERIAL, ident);
            if (comm == null) comm = Profile.getComm(type, DEVICE_INTERFACE_Types.DEVICE_INTERFACE_BLE, ident);
            if (comm == null) comm = Profile.getComm(type, DEVICE_INTERFACE_Types.DEVICE_INTERFACE_AUDIO_JACK, ident);
            if (comm == null) comm = Profile.getComm(type, DEVICE_INTERFACE_Types.DEVICE_INTERFACE_IP, ident);
            DEVICE_INTERFACE_Types connect = DEVICE_INTERFACE_Types.DEVICE_INTERFACE_UNKNOWN;
            if (comm != null)
            {
                connect = comm.getDeviceConnection();
            }


            if (comm != null)
            {
                if (comm.isKB && state == DeviceState.Connected)
                {
                    MessageBox.Show("KB-Mode device detected.  This application is for HID-Mode devices only. Please put your device into HID-Mode and try again", "KB Mode Device Detected", MessageBoxButtons.OK, MessageBoxIcon.Error);
                    Profile.closeDevice(comm.ident);
                }
                if (comm.isKB) return;
            }
            switch (state)
            {


                case DeviceState.DefaultDeviceTypeChange:
                    output("\nSDK Default Device =  " + IDTechSDK.Profile.IDT_DEVICE_String(type, connect),ident);
             


                    break;
                case DeviceState.Connected:
                    connectlabel( "Connected " + IDTechSDK.Profile.IDT_DEVICE_String(type, connect, ident),ident);      
                    break;
                case DeviceState.Disconnected:
                    connectlabel( "Disconnected " + IDTechSDK.Profile.IDT_DEVICE_String(type, connect, ident),ident);
                    break;
                case DeviceState.ConnectionFailed:
                    break;
                case DeviceState.RKI:
                    break;

                case DeviceState.SocketEstablished:
                    break;
                case DeviceState.SocketFailed:
                    break;
                case DeviceState.ViVOconfig:
                    break;
                case DeviceState.TransactionData:


                    if (cardData == null)
                    {
                        output("Card Data Null - Breaking", ident);
                        break;
                    }

                    output("cardData.emv_resultCode = " + cardData.emv_resultCode.ToString(), ident);



                    if (cardData.Event == EVENT_TRANSACTION_DATA_Types.EVENT_TRANSACTION_PIN_DATA ||
                        cardData.Event == EVENT_TRANSACTION_DATA_Types.EVENT_TRANSACTION_DATA_CARD_DATA ||
                        cardData.Event == EVENT_TRANSACTION_DATA_Types.EVENT_TRANSACTION_DATA_EMV_DATA)

                    {

                        displayCardData(cardData, ident);
                    }

                    break;

                case DeviceState.EMVCallback:


                    if (emvCallback == null) break;
                    if (emvCallback.callbackType == EMV_CALLBACK_TYPE.EMV_CALLBACK_TYPE_LCD) //LCD Callback Type
                    {
                        if (emvCallback.lcd_displayMode == EMV_LCD_DISPLAY_MODE.EMV_LCD_DISPLAY_MODE_CLEAR_SCREEN)
                        {

                            return;
                        }
                        else if (emvCallback.lcd_displayMode == EMV_LCD_DISPLAY_MODE.EMV_LCD_DISPLAY_MODE_MESSAGE || emvCallback.lcd_displayMode == EMV_LCD_DISPLAY_MODE.EMV_LCD_DISPLAY_MODE_AR_MESSAGE2 || emvCallback.lcd_displayMode == EMV_LCD_DISPLAY_MODE.EMV_LCD_DISPLAY_MODE_AR_MESSAGE3)
                        {
                            showMessageOnLCD(emvCallback.language, emvCallback.lcd_messages, ident);
                        }
                        else if (emvCallback.lcd_displayMode == EMV_LCD_DISPLAY_MODE.EMV_LCD_DISPLAY_MODE_AR_MESSAGE4)
                        {
                            showMessageOnLCD(emvCallback.language, emvCallback.lcd_messages, ident);

                            output("Mode 4 Callback - Awaiting Response.  \r\n", ident);
                            RETURN_CODE RT = IDT_Device.SharedController.emv_callbackResponseLCD(EMV_LCD_DISPLAY_MODE.EMV_LCD_DISPLAY_MODE_AR_MESSAGE4, 0);

                        }


                    }



                    break;
            }

        }




        private void displayCardData(IDTTransactionData cardData, string ident)
        {
            string text = "";
            if (cardData != null && cardData.Base64 != null && cardData.Base64.Length > 0)
                text += "Raw Data (Base 64):\r\n" + cardData.Base64 + "\r\n\r\n";
            if (cardData != null && cardData.fastEMV != null && cardData.fastEMV.Length > 0)
                text += "Fast EMV Data: " + cardData.fastEMV + "\r\n\r\n";
            if (cardData.Event == EVENT_TRANSACTION_DATA_Types.EVENT_TRANSACTION_PIN_DATA)
            {
                output("PIN Data received:\r\n", ident);
                if (cardData.pin_KSN != null && cardData.pin_KSN.Length > 0)
                {
                    output("KSN: " + cardData.pin_KSN + "\r\n", ident);

                }

                if (cardData.pin_pinblock != null && cardData.pin_pinblock.Length > 0)
                {
                    output("PINBLOCK: " + cardData.pin_pinblock + "\r\n", ident);

                }

                if (cardData.pin_KeyEntry != null && cardData.pin_KeyEntry.Length > 0)
                {
                    output("Key Entry: " + cardData.pin_KeyEntry + "\r\n", ident);

                }

                return;

            }

            if (cardData.Event == EVENT_TRANSACTION_DATA_Types.EVENT_TRANSACTION_DATA_CARD_DATA)
            {
                if (cardData.msr_rawData != null)
                {

                    output("Raw Data (HEX): \r\n" + Common.getHexStringFromBytes(cardData.msr_rawData) + "\r\n", ident);

                }

            }

            if (cardData.captured_KSN != null)
            {
                //encrypted captured data
                if (cardData.captured_InitialVector != null)
                    text += "Man. Captured Initial Vector: " +
                            Common.getHexStringFromBytes(cardData.captured_InitialVector) + "\r\n";
                if (cardData.captured_KSN != null)
                    text += "Man. Captured KSN: " + Common.getHexStringFromBytes(cardData.captured_KSN) + "\r\n";
                if (cardData.captured_PAN != null)
                    text += "Man. Captured Encrypted PAN: " + Common.getHexStringFromBytes(cardData.captured_PAN) +
                            "\r\n";
                if (cardData.captured_Expiry != null)
                    text += "Man. Captured Encrypted Expiry: " +
                            Common.getHexStringFromBytes(cardData.captured_Expiry) + "\r\n";
                if (cardData.captured_CSC != null)
                    text += "Man. Captured Encrypted CSC: " + Common.getHexStringFromBytes(cardData.captured_CSC) +
                            "\r\n";
                if (cardData.captured_firstPANDigits != null && cardData.captured_firstPANDigits.Length > 0)
                    text += "PAN First Digits: " + cardData.captured_firstPANDigits + "\r\n";
                if (cardData.captured_lastPANDigits != null && cardData.captured_firstPANDigits.Length > 0)
                    text += "PAN Last Digits: " + cardData.captured_lastPANDigits + "\r\n";
                if (cardData.captured_MACValue != null)
                    text += "Man. Captured MAC: " + Common.getHexStringFromBytes(cardData.captured_MACValue) + "\r\n";
                if (cardData.captured_MACKSN != null)
                    text += "Man. Captured MAC KSN: " + Common.getHexStringFromBytes(cardData.captured_MACKSN) + "\r\n";




            }
            else if (cardData.captured_PAN != null)
            {
                //unecnrypted captured data
                if (cardData.captured_PAN != null)
                    text += "Man. Captured  PAN: " + System.Text.Encoding.UTF8.GetString(cardData.captured_PAN) +
                            "\r\n";
                if (cardData.captured_Expiry != null)
                    text += "Man. Captured  Expiry: " + System.Text.Encoding.UTF8.GetString(cardData.captured_Expiry) +
                            "\r\n";
                if (cardData.captured_CSC != null)
                    text += "Man. Captured  CSC: " + System.Text.Encoding.UTF8.GetString(cardData.captured_CSC) +
                            "\r\n";
            }



            if (cardData.device_RSN != null && cardData.device_RSN.Length > 0)
                text += "Serial Number: " + cardData.device_RSN + "\r\n";

            if (cardData.msr_track1Length > 0)
                text += "Track 1: " + cardData.msr_track1 + "\r\n";

            if (cardData.msr_encTrack1 != null)
                text += "Track 1 Encrypted: " + Common.getHexStringFromBytes(cardData.msr_encTrack1) + "\r\n";



            if (cardData.msr_hashTrack1 != null)
                text += "Track 1 Hash: " + Common.getHexStringFromBytes(cardData.msr_hashTrack1) + "\r\n";




            if (cardData.msr_track2Length > 0)
                text += "Track 2: " + cardData.msr_track2 + "\r\n";


            if (cardData.msr_encTrack2 != null)
                text += "Track 2 Encrypted: " + Common.getHexStringFromBytes(cardData.msr_encTrack2) + "\r\n";


            if (cardData.msr_hashTrack2 != null)
                text += "Track 2 Hash: " + Common.getHexStringFromBytes(cardData.msr_hashTrack2) + "\r\n";




            if (cardData.msr_track3Length > 0)
                text += "Track 3: " + cardData.msr_track3 + "\r\n";


            if (cardData.msr_encTrack3 != null)
                text += "Track 3 Encrypted: " + Common.getHexStringFromBytes(cardData.msr_encTrack3) + "\r\n";


            if (cardData.msr_hashTrack3 != null)
                text += "Track 3 Hash: " + Common.getHexStringFromBytes(cardData.msr_hashTrack3) + "\r\n";

            if (cardData.msr_KSN != null)
                text += "KSN: " + Common.getHexStringFromBytes(cardData.msr_KSN) + "\r\n";


            if (cardData.emv_clearingRecord != null)
            {
                if (cardData.emv_clearingRecord.Length > 0)
                {
                    text += "\r\nCTLS Clearing Record: \r\n";
                    text += Common.getHexStringFromBytes(cardData.emv_clearingRecord) + "\r\n";
                    Dictionary<string, string> dict = Common.processTLVUnencrypted(cardData.emv_clearingRecord);
                    foreach (KeyValuePair<string, string> kvp in dict) text += kvp.Key + ": " + kvp.Value + "\r\n";
                    text += "\r\n\r\n";

                }

            }

            if (cardData.emv_unencryptedTags != null)
            {
                if (cardData.emv_unencryptedTags.Length > 0)
                {
                    text += "\r\n======================== \r\n";

                    text += "\r\nUnencrypted Tags: \r\n";
                    text += Common.getHexStringFromBytes(cardData.emv_unencryptedTags) + "\r\n\r\n";
                    text += tlvToValues(cardData.emv_unencryptedTags);
                    text += "\r\n======================== \r\n";
                }

            }

            if (cardData.emv_encryptedTags != null)
            {
                if (cardData.emv_encryptedTags.Length > 0)
                {
                    text += "\r\n======================== \r\n";
                    text += "\r\nEncrypted Tags: \r\n";
                    text += Common.getHexStringFromBytes(cardData.emv_encryptedTags) + "\r\n\r\n";
                    text += tlvToValues(cardData.emv_encryptedTags);
                    text += "\r\n======================== \r\n";

                }

            }

            if (cardData.emv_maskedTags != null)
            {
                if (cardData.emv_maskedTags.Length > 0)
                {
                    text += "\r\n======================== \r\n";
                    text += "\r\nMasked Tags: \r\n";
                    text += Common.getHexStringFromBytes(cardData.emv_maskedTags) + "\r\n\r\n";
                    text += tlvToValues(cardData.emv_maskedTags);
                    text += "\r\n======================== \r\n";

                }

            }

            if (cardData.emv_hasAdvise) text += "CARD RESPONSE HAS ADVISE" + "\r\n";
            if (cardData.emv_hasReversal) text += "CARD RESPONSE HAS REVERSAL" + "\r\n";
            if (cardData.iccPresent == 1) text += "ICC Present: TRUE" + "\r\n";
            if (cardData.iccPresent == 2) text += "ICC Present: FALSE" + "\r\n";
            if (cardData.isCTLS == 1) text += "CTLS Capture: TRUE" + "\r\n";
            if (cardData.isCTLS == 2) text += "CTLS Capture: FALSE" + "\r\n";


            if (cardData.msr_extendedField != null && cardData.msr_extendedField.Length > 0)
                text += "Extended Field Bytes: " + Common.getHexStringFromBytes(cardData.msr_extendedField) + "\r\n";


            if (cardData.captureEncryptType == CAPTURE_ENCRYPT_TYPE.CAPTURE_ENCRYPT_TRANS_ARMOR_PKI)
                text += "Encryption Type: TRANS_ARMOR_PKI\r\n";
            if (cardData.captureEncryptType == CAPTURE_ENCRYPT_TYPE.CAPTURE_ENCRYPT_TRANS_ARMOR_TDES)
                text += "Encryption Type: TRANS_ARMOR_TDES\r\n";
            if (cardData.captureEncryptType == CAPTURE_ENCRYPT_TYPE.CAPTURE_ENCRYPT_VOLTAGE)
                text += "Encryption Type: VOLTAGE\r\n";
            if (cardData.captureEncryptType == CAPTURE_ENCRYPT_TYPE.CAPTURE_ENCRYPT_VISA_FPE)
                text += "Encryption Type: VISA_FPE\r\n";
            if (cardData.captureEncryptType == CAPTURE_ENCRYPT_TYPE.CAPTURE_ENCRYPT_VERIFONE_FPE)
                text += "Encryption Type: VERIFONE_FPE\r\n";
            if (cardData.captureEncryptType == CAPTURE_ENCRYPT_TYPE.CAPTURE_ENCRYPT_DESJARDIN)
                text += "Encryption Type: DESJARDIN\r\n";

            if (cardData.captureEncryptType == CAPTURE_ENCRYPT_TYPE.CAPTURE_ENCRYPT_TYPE_NONE)
                text += "Encryption Type: NONE\r\n";

            if (cardData.captureEncryptType != CAPTURE_ENCRYPT_TYPE.CAPTURE_ENCRYPT_TYPE_NONE)
            {
                if (cardData.msr_keyVariantType == KEY_VARIANT_TYPE.KEY_VARIANT_TYPE_DATA)
                    text += "Key Type: Data Variant\r\n";
                else if (cardData.msr_keyVariantType == KEY_VARIANT_TYPE.KEY_VARIANT_TYPE_PIN)
                    text += "Key Type: PIN Variant\r\n";
            }



            if (cardData.mac != null)
                text += "MAC: " + Common.getHexStringFromBytes(cardData.mac) + "\r\n";

            if (cardData.macKSN != null)
                text += "MAC KSN: " + Common.getHexStringFromBytes(cardData.macKSN) + "\r\n";

            if (cardData.msr_sessionID != null)
                text += "Session ID: " + Common.getHexStringFromBytes(cardData.msr_sessionID) + "\r\n";

            if (cardData.Event == EVENT_TRANSACTION_DATA_Types.EVENT_TRANSACTION_DATA_EMV_DATA)
            {
                switch (cardData.emv_resultCode)
                {
                    case EMV_RESULT_CODE.EMV_RESULT_CODE_APPROVED:
                        text += ("EMV RESULT: " + "EMV_RESULT_CODE_APPROVED" + "\r\n");
                        output("APPROVED", ident);
                        break;
                    case EMV_RESULT_CODE.EMV_RESULT_CODE_APPROVED_OFFLINE:
                        output("APPROVED", ident);
                        text += ("EMV RESULT: " + "EMV_RESULT_CODE_APPROVED_OFFLINE" + "\r\n");
                        break;
                    case EMV_RESULT_CODE.EMV_RESULT_CODE_DECLINED_OFFLINE:
                        text += ("EMV RESULT: " + "EMV_RESULT_CODE_DECLINED_OFFLINE" + "\r\n");
                        break;
                    case EMV_RESULT_CODE.EMV_RESULT_CODE_DECLINED:
                        text += ("EMV RESULT: " + "EMV_RESULT_CODE_DECLINED" + "\r\n");
                        break;
                    case EMV_RESULT_CODE.EMV_RESULT_CODE_GO_ONLINE:
                        text += ("EMV RESULT: " + "EMV_RESULT_CODE_GO_ONLINE" + "\r\n");
                        break;
                    case EMV_RESULT_CODE.EMV_RESULT_CODE_CALL_YOUR_BANK:
                        output("CALL YOUR BANK", ident);
                        text += ("EMV RESULT: " + "EMV_RESULT_CODE_CALL_YOUR_BANK" + "\r\n");
                        break;
                    case EMV_RESULT_CODE.EMV_RESULT_CODE_NOT_ACCEPTED:
                        text += ("EMV RESULT: " + "EMV_RESULT_CODE_NOT_ACCEPTED" + "\r\n");
                        break;
                    case EMV_RESULT_CODE.EMV_RESULT_CODE_FALLBACK_TO_MSR:
                        text += ("EMV RESULT: " + "EMV_RESULT_CODE_FALLBACK_TO_MSR" + "\r\n");
                        output("", ident);
                        output(getLCDDisplayMessage(new byte[] { 0x45, 0x4E }, (int)DisplayMessages.DISPLAY_MESSAGE_IDENTIFIER.MSG_NEW_WELCOME), ident);
                        break;
                    case EMV_RESULT_CODE.EMV_RESULT_CODE_TIMEOUT:
                        text += ("EMV RESULT: " + "EMV_RESULT_CODE_TIMEOUT" + "\r\n");
                        break;
                    case EMV_RESULT_CODE.EMV_RESULT_CODE_AUTHENTICATE_TRANSACTION:
                        text += ("EMV RESULT: " + "EMV_RESULT_CODE_AUTHENTICATE_TRANSACTION" + "\r\n");
                        break;
                    case EMV_RESULT_CODE.EMV_RESULT_CODE_SWIPE_NON_ICC:
                        text += ("EMV RESULT: " + "EMV_RESULT_CODE_SWIPE_NON_ICC" + "\r\n");
                        break;
                    case EMV_RESULT_CODE.EMV_RESULT_CODE_CTLS_TWO_CARDS:
                        text += ("EMV RESULT: " + "EMV_RESULT_CODE_CTLS_TWO_CARDS" + "\r\n");
                        break;
                    case EMV_RESULT_CODE.EMV_RESULT_CODE_CTLS_TERMINATE:
                        text += ("EMV RESULT: " + "EMV_RESULT_CODE_CTLS_TERMINATE" + "\r\n");
                        break;
                    case EMV_RESULT_CODE.EMV_RESULT_CODE_CTLS_TERMINATE_TRY_ANOTHER:
                        text += ("EMV RESULT: " + "EMV_RESULT_CODE_CTLS_TERMINATE_TRY_ANOTHER" + "\r\n");
                        break;
                    case EMV_RESULT_CODE.EMV_RESULT_CODE_GO_ONLINE_CTLS:
                        text += ("EMV RESULT: " + "EMV_RESULT_CODE_GO_ONLINE_CTLS" + "\r\n");
                        break;
                    case EMV_RESULT_CODE.EMV_RESULT_CODE_FAILED:
                        text += ("EMV RESULT: " + "EMV_RESULT_CODE_FAILED" + "\r\n");
                        break;
                    case EMV_RESULT_CODE.EMV_RESULT_CODE_MSR_SWIPE_CAPTURED:
                        text += ("EMV RESULT: " + "EMV_RESULT_CODE_MSR_SWIPE_CAPTURED" + "\r\n");
                        break;
                    case EMV_RESULT_CODE.EMV_RESULT_CODE_REQUEST_ONLINE_PIN:
                        text += ("EMV RESULT: " + "EMV_RESULT_CODE_REQUEST_ONLINE_PIN" + "\r\n");
                        break;
                    case EMV_RESULT_CODE.EMV_RESULT_CODE_REQUEST_SIGNATURE:
                        text += ("EMV RESULT: " + "EMV_RESULT_CODE_REQUEST_SIGNATURE" + "\r\n");
                        break;
                    case EMV_RESULT_CODE.EMV_RESULT_CODE_ADVISE_REQUIRED:
                        text += ("EMV RESULT: " + "EMV_RESULT_CODE_ADVISE_REQUIRED" + "\r\n");
                        break;
                    case EMV_RESULT_CODE.EMV_RESULT_CODE_REVERSAL_REQUIRED:
                        text += ("EMV RESULT: " + "EMV_RESULT_CODE_REVERSAL_REQUIRED" + "\r\n");
                        break;
                    case EMV_RESULT_CODE.EMV_RESULT_CODE_ADVISE_REVERSAL_REQUIRED:
                        text += ("EMV RESULT: " + "EMV_RESULT_CODE_ADVISE_REVERSAL_REQUIRED" + "\r\n");
                        break;
                    case EMV_RESULT_CODE.EMV_RESULT_CODE_NO_ADVISE_REVERSAL_REQUIRED:
                        text += ("EMV RESULT: " + "EMV_RESULT_CODE_NO_ADVISE_REVERSAL_REQUIRED" + "\r\n");
                        break;
                    default:
                        text += ("EMV RESULT: " + errorCode.getErrorString((RETURN_CODE)cardData.emv_resultCode) +
                                 "\r\n");
                        break;
                }
            }

            if (cardData.emv_transaction_Error_Code > 0)
                text += ("Transaction Error: " + errorCode.getTransError(cardData.emv_transaction_Error_Code) + "\r\n");
            if (cardData.emv_RF_State > 0)
                text += ("RF State: " + errorCode.getRFState(cardData.emv_RF_State) + "\r\n");
            if (cardData.emv_ESC > 0)
                text += ("Extended Status Code: " + errorCode.getExtendedStatusCode(cardData.emv_ESC) + "\r\n");
            if (cardData.emv_appErrorFn > 0)
                text += ("App Error Function: " + errorCode.getEMVAppErrorFn(cardData.emv_appErrorFn) + "\r\n");
            if (cardData.emv_appErrorState > 0)
                text += ("App Error State: " + errorCode.getEMVAppErrorState(cardData.emv_appErrorState) + "\r\n");



            if (cardData.ctlsApplication > 0)
            {
                text += "Contactless Application: ";
                switch (cardData.ctlsApplication)
                {
                    case CTLS_APPLICATION.CTLS_APPLICATION_AMEX:
                        text += ("CTLS_APPLICATION_AMEX" + "\r\n");
                        break;
                    case CTLS_APPLICATION.CTLS_APPLICATION_DISCOVER:
                        text += ("CTLS_APPLICATION_DISCOVER" + "\r\n");
                        break;
                    case CTLS_APPLICATION.CTLS_APPLICATION_MASTERCARD:
                        text += ("CTLS_APPLICATION_MASTERCARD" + "\r\n");
                        break;
                    case CTLS_APPLICATION.CTLS_APPLICATION_VISA:
                        text += ("CTLS_APPLICATION_VISA" + "\r\n");
                        break;
                    case CTLS_APPLICATION.CTLS_APPLICATION_SPEEDPASS:
                        text += ("CTLS_APPLICATION_SPEEDPASS" + "\r\n");
                        break;
                    case CTLS_APPLICATION.CTLS_APPLICATION_GIFT_CARD:
                        text += ("CTLS_APPLICATION_GIFT_CARD" + "\r\n");
                        break;
                    case CTLS_APPLICATION.CTLS_APPLICATION_DINERS_CLUB:
                        text += ("CTLS_APPLICATION_DINERS_CLUB" + "\r\n");
                        break;
                    case CTLS_APPLICATION.CTLS_APPLICATION_EN_ROUTE:
                        text += ("CTLS_APPLICATION_EN_ROUTE" + "\r\n");
                        break;
                    case CTLS_APPLICATION.CTLS_APPLICATION_JCB:
                        text += ("CTLS_APPLICATION_JCB" + "\r\n");
                        break;
                    case CTLS_APPLICATION.CTLS_APPLICATION_VIVO_DIAGNOSTIC:
                        text += ("CTLS_APPLICATION_VIVO_DIAGNOSTIC" + "\r\n");
                        break;
                    case CTLS_APPLICATION.CTLS_APPLICATION_HID:
                        text += ("CTLS_APPLICATION_HID" + "\r\n");
                        break;
                    case CTLS_APPLICATION.CTLS_APPLICATION_MSR_SWIPE:
                        text += ("CTLS_APPLICATION_MSR_SWIPE" + "\r\n");
                        break;
                    case CTLS_APPLICATION.CTLS_APPLICATION_RESERVED:
                        text += ("CTLS_APPLICATION_RESERVED" + "\r\n");
                        break;
                    case CTLS_APPLICATION.CTLS_APPLICATION_DES_FIRE_TRACK_DATA:
                        text += ("CTLS_APPLICATION_DES_FIRE_TRACK_DATA" + "\r\n");
                        break;
                    case CTLS_APPLICATION.CTLS_APPLICATION_DES_FIRE_RAW_DATA:
                        text += ("CTLS_APPLICATION_DES_FIRE_RAW_DATA" + "\r\n");
                        break;
                    case CTLS_APPLICATION.CTLS_APPLICATION_RBS:
                        text += ("CTLS_APPLICATION_RBS" + "\r\n");
                        break;
                    case CTLS_APPLICATION.CTLS_APPLICATION_VIVO_COMM:
                        text += ("CTLS_APPLICATION_VIVO_COMM" + "\r\n");
                        break;

                }
            }



            output(text, ident);

            text = "";
            byte[] temp = new byte[0];




        }

        private string tlvToValues(byte[] tlv)
        {
            string text = "";
            Dictionary<string, string> dict = Common.processTLVUnencrypted(tlv);
            foreach (KeyValuePair<string, string> kvp in dict) text += kvp.Key + ": " + kvp.Value + "\r\n";
            dict.Clear();
            dict = null;
            return text;
        }


        private void showMessageOnLCD(byte[] language, byte[] message, string ident)
        {
            if (message == null) return;
            StringBuilder sb = new StringBuilder();
            int position = 0;
            while (position < message.Length)
            {
                byte line = message[position++];
                if (position < message.Length)
                {
                    if ((line & 0x80) == 0x80)
                    {
                        sb.Append(getLCDDisplayMessage(language, message[position++]));

                    }

                    position++;
                    if (position < message.Length)
                        sb.AppendLine();

                }


            }


            System.Diagnostics.Debug.WriteLine("Setting LCD Screen Message");
            output("\r\n" + sb.ToString() + "\r\n", ident);


        }

        public static string getLCDDisplayMessage(byte[] language, int id)
        {
            if (id > (int)DisplayMessages.DISPLAY_MESSAGE_IDENTIFIER.TRANS_MSG_NEW_TBL_LIMIT) return "";
            ASCIIEncoding encoding = new ASCIIEncoding();
            string line1 = "";
            string line2 = "";
            string sLanguage = encoding.GetString(language, 0, 2).ToUpper();
            DisplayMessages.DISPLAY_MESSAGE_LANGUAGE theLang = DisplayMessages.DISPLAY_MESSAGE_LANGUAGE.ENGLISH;
            if (sLanguage.Equals("FR"))
            {
                theLang = DisplayMessages.DISPLAY_MESSAGE_LANGUAGE.FRENCH;
            }
            if (sLanguage.Equals("EF"))
            {
                theLang = DisplayMessages.DISPLAY_MESSAGE_LANGUAGE.ENGLISH_FRENCH;
            }
            if (sLanguage.Equals("ES"))
            {
                theLang = DisplayMessages.DISPLAY_MESSAGE_LANGUAGE.SPANISH;
            }
            if (sLanguage.Equals("PT"))
            {
                theLang = DisplayMessages.DISPLAY_MESSAGE_LANGUAGE.PORTUGUESE;
            }
            if (sLanguage.Equals("ZH"))
            {
                theLang = DisplayMessages.DISPLAY_MESSAGE_LANGUAGE.CHINESE;
            }
            if (sLanguage.Equals("JA"))
            {
                theLang = DisplayMessages.DISPLAY_MESSAGE_LANGUAGE.JAPANESE;
            }

            DisplayMessages.retrieveMessage(theLang, (DisplayMessages.DISPLAY_MESSAGE_IDENTIFIER)id, ref line1, ref line2);
            return (line1 + "\r\n" + line2);
        }
        WorldNet wn = null;
        private void startTrans_Click(object sender, EventArgs e)
        {
            if (wn == null) wn = new WorldNet();

            WorldNet.WorldNetData data = new WorldNet.WorldNetData();

            data.terminal = "4480001";  //WorldNet Terminal (USD)
            data.apiKey = "afae1c3fa41612acca86b435f65edcf3c930e3fe84b4dab4ef14321eb242890e09919c6e45e0a0506f523db0b5e6da2589c5182b1aca06b91558100619576e88";
                   //WorldNet Merchant API Key
            data.transactionType = WorldNet.TRANSACTION_TYPE.TRANSACTION_TYPE_SALE;   //specify Sale, Pre-Auth, or Completion
                                                                                      // data.transactionType =  WorldNet.TRANSACTION_TYPE.TRANSACTION_TYPE_PRE_AUTH;   //specify Sale, Pre-Auth, or Completion
                                                                                      // data.transactionType =  WorlWorldNetdNet.TRANSACTION_TYPE.TRANSACTION_TYPE_COMPLETION;   //specify Sale, Pre-Auth, or Completion
                                                                                      // data.transactionType =  WorldNet.TRANSACTION_TYPE.TRANSACTION_TYPE_VOID;   //specify Sale, Pre-Auth, or Completion
                                                                                      // data.transactionType =  WorldNet.TRANSACTION_TYPE.TRANSACTION_TYPE_RETURN;   //specify Sale, Pre-Auth, or Completion
            data.amount = "32.00";   //specify transaction Amount
            data.timeout = 30;  //Timeout for transaction
            data.isTest = true;  //if test transaction, set to TRUE.  If live transactions, set to FALSE
            data.transactionID = ""; //required if COMPLETION, VOID or RETURN.  Otherwise, leave blank. For WorldNet, this is "uniqueReference"
            data.orderID = ""; //required value.  If blank, SDK will put in random number. 
            data.deviceID = "";  //set to device identifier to target specific device, or leave empty to target first/only device.
            data.Operator = "";  //operator for Completion, Void or Return. Blank will put value "VP3300"
            data.refundReason = "";  //for Refund, must provide a reason.  If blank, tranaction ID will be used  
            data.additionalTags = "";  //Used for passing additional tags with the transaction request  
            wn.returnRequest = true; //set to TRUE if you would like the request packet returned with the response

            bool requestOnly = false; // If this is TRUE, the SDK will return the request packet instead of contacting WorldNet with it. For integrations that want to control communication with WorldNet servers

            string ident = data.deviceID; // If you want to target a specific device, you specify either as a parameter you pass to executeTransaction, or through data.deviceID. Either method is acceptable 

            RETURN_CODE rt = wn.executeTransaction(data, WorldNetCallback, requestOnly, ident);

            if (rt == RETURN_CODE.RETURN_CODE_DO_SUCCESS)
            {
                output("Start Transaction Successful\r\n", IDT_Device.currentIdent());
            }
            else
            {
                output("Start transaction failed Error Code: " + "0x" + String.Format("{0:X}", (ushort)rt) +
                                    ": " + IDTechSDK.errorCode.getErrorString(rt) + "\r\n", IDT_Device.currentIdent());

            }

        }



        void WorldNetCallback(IDT_DEVICE_Types sender, string jsonResponse, RETURN_CODE resultCode, EMV_RESULT_CODE emvCode, string ident, IDTTransactionData cardData)
        {
            if (InvokeRequired)
            {
                Invoke((MethodInvoker)delegate { WorldNetCallback(sender, jsonResponse, resultCode, emvCode, ident, cardData); });
                return;
            }
            if (cardData != null)
            {
                output("\r\n\r\n", ident);
                displayCardData(cardData, ident);
            }
            output("\r\n\r\nResponse received from WorldNet: " + resultCode.ToString() + "\r\n\r\n", ident);
            output(jsonResponse, ident);
        }

        private void cancelTrans_Click(object sender, EventArgs e)
        {
            if (wn == null) return;
            RETURN_CODE rt = wn.cancelTransaction();

            if (rt == RETURN_CODE.RETURN_CODE_DO_SUCCESS)
            {
                output("Cancel Transaction Successful\r\n", IDT_Device.currentIdent());
            }
            else
            {
                output("Cancel transaction failed Error Code: " + "0x" + String.Format("{0:X}", (ushort)rt) +
                                    ": " + IDTechSDK.errorCode.getErrorString(rt) + "\r\n", IDT_Device.currentIdent());
            }
        }

        private void getfimrware_Click(object sender, EventArgs e)
        {
            string fwver = "";
            RETURN_CODE rt = IDT_Device.SharedController.device_getFirmwareVersion(ref fwver);

            if (rt == RETURN_CODE.RETURN_CODE_DO_SUCCESS)
            {
                output("Firmware Version: " + fwver, IDT_Device.currentIdent());
            }
            else
            {
                output("Firmware version failed Error Code: " + "0x" + String.Format("{0:X}", (ushort)rt) +
                                    ": " + IDTechSDK.errorCode.getErrorString(rt) + "\r\n", IDT_Device.currentIdent());
            }
        }
    }
}
